<?php

class Login extends CI_Controller {
    private function is_valid_file($rel_path = '/', $file_html = 'home.html')
    {
        $abs_path = APPPATH.$rel_path.$file_html;
        if (file_exists($abs_path))
            return true;

        return false;
    }

    private function is_post_valid()
    {
        return $this->form_validation->run();
    }

    private function is_username_valid($username = NULL, $db_username = NULL)
    {
        if (!$username || !$db_username)
            return false;

        return !strcmp($username, $db_username);
    }

    private function is_password_valid($password = NULL, $db_password = NULL)
    {
        if (!$password || !$db_password)
            return false;

        $pepp = getenv('HOST_PEPPER');
        $pepp_password = hash_hmac("sha256", $password, $pepp);

        return password_verify($pepp_password, $db_password);
    }

    private function is_login_valid($db_username = NULL, $username = NULL,
                                    $db_password = NULL, $password = NULL)
    {
        if (!$db_username || !$db_password)
            return false;

        if (!$this->is_username_valid($username, $db_username))
            return false;
        
        if (!$this->is_password_valid($password, $db_password))
            return false;

        return true;
    }

    private function view($file_name = 'login')
    {
        $file_html = $file_name.'.html';
        if ($this->is_valid_file('views/page/', $file_html))
        {
            $this->form_validation->set_rules('username', 'Username', 'required',
                array('required' => 'You must provide a %s')
            );

            $this->form_validation->set_rules('password', 'Password', 'required',
                array('required' => 'You must provide a %s')
            );

            if ($this->is_post_valid())
            {
                $username = $this->input->post('username');
                $password = $this->input->post('password');

                /* Add backticks on ientifiers */
                $this->db->protect_identifiers('users', TRUE);
                $this->db->protect_identifiers('username', TRUE);
                $this->db->protect_identifiers('password', TRUE);

                /* Always use query bindings as they are automatically escaped */
                $stmt = "SELECT username,password FROM users WHERE username = ?";
                $db_data = $this->db->query($stmt, $username);

                /* Return an array of row objects, empty array on failure */
                $db_data = $db_data->result();
                if ($db_data && $db_data[0])
                {
                    $db_username = $db_data[0]->username;
                    $db_password = $db_data[0]->password;
                    if ($this->is_login_valid($db_username, $username,
                                              $db_password, $password))
                    {
                        /* Initialize session data */
                        $this->form_validation->set_string('Login successful');
                        $_SESSION['username'] = $db_username;
                    }
                    else
                    {
                        //Debug:

                        //$error = $this->db->error();
                        //var_dump($error);

                        $this->form_validation->set_message(
                            'submit_msg', 'Didn\'t work, :|');
                    }
                }
                else
                {
                    $this->form_validation->set_string('I am empty, -.-');
                }
            }

            /* Validation errors already set, if any */
            $this->load->view('page/'.$file_html);
        }
        else
        {
            redirect(base_url('index.html'));
        }
    }

    public function auth($param = NULL)
    {
        $username = isset($_POST['username']) ? $_POST['username'] : NULL;
        $password = isset($_POST['password']) ? $_POST['password'] : NULL;
        if (!$username || !$password)
        {
            $this->output->set_status_header(404);
            return;
        }

        /* 1 for active user */
        $status = 1;

        /* Add backticks on ientifiers */
        $this->db->protect_identifiers('users', TRUE);
        $this->db->protect_identifiers('username', TRUE);
        $this->db->protect_identifiers('password', TRUE);

        /* Always use query bindings as they are automatically escaped */
        $stmt = "SELECT username,password FROM users WHERE username = ? AND status = ?";
        $db_data = $this->db->query($stmt, [$username, $status]);

        /* Return an array of row objects, empty array on failure */
        $db_data = $db_data->result();
        if ($db_data && $db_data[0])
        {
            $db_username = $db_data[0]->username;
            $db_password = $db_data[0]->password;
            if ($this->is_login_valid($db_username, $username,
                                      $db_password, $password))
            {
                /* Initialize session data */
                $this->output->set_status_header(204);
                $_SESSION['username'] = $db_username;

                return;
            }
        }

        $this->output->set_status_header(404);
    }
}
